import CSLib.*;
import java.awt.*;

public class Clock {
  // Maintain a clock.
  // Author: Courtney Mickunas, November 21, 2000
  
  int hour,
      minute;

  public Clock () {
    hour = 12;
    minute = 0;
  }

  public Clock (int h, int m) {
    hour = h;
    minute = m;
  }

  // Obtain values for hour and minute from user input.
  public void getData () {
    InputBox in = new InputBox();
    in.setPrompt("What is the hour?");
    setHour(in.readInt());    
    in.setPrompt("What is the minute?");
    setMinute(in.readInt());    
  }

  // Return the current time as a string.
  public String toString () { return (hour + ":" + minute); }

  public void setHour (int h) { set(h, minute); }
  public void setMinute (int m) { set(hour, m); }

  public void set (int h, int m) {
    int totalMinutes = (hour * 60 + minute) % (12 * 60);
    // totalMinutes is between -(12 * 60) and (12 * 60)
    if (totalMinutes < 0) totalMinutes = totalMinutes + (12 * 60);
    hour = totalMinutes / 60;
    if (hour == 0) hour = 12;
    minute = totalMinutes % 60;
  }

  public int getHour () { return hour; }
  public int getMinute () { return minute; }

  // Compare the time of this clock to another.
  // Return true if this time is less than
  // the time of the other clock.
  public boolean priorTo (Clock c) {
    return (hour < c.hour ||
            hour == c.hour && minute < c.minute
           );
  }

  // Draw a clock with radius r in the center of the DrawingBox
  public void display (DrawingBox d, int r) {
    int x, y;
    x = d.getDrawableWidth()/2;
    y = d.getDrawableHeight()/2;

    double theta;    // angle from 12:00
    int x1, y1;      // end point of a clock hand

    // Draw the clock in black.
    d.setColor(Color.black);
    d.drawOval(x-r, y-r, 2*r, 2*r);

    // Set the color to blue for the hands
    d.setColor(Color.blue);
    // Draw the minute hand.
    theta = 2*Math.PI*minute/60.0;
    x1 = x + (int)(r*Math.sin(theta));
    y1 = y - (int)(r*Math.cos(theta));
    d.drawLine(x, y, x1, y1);

    // Draw the hour hand.
    theta = 2*Math.PI*(hour+minute/60.0)/12.0;
    x1 = x + (int)(r*.8*Math.sin(theta));
    y1 = y - (int)(r*.8*Math.cos(theta));
    d.drawLine(x, y, x1, y1);
  }

  // Draw a clock with radius r at (x,y) in the DrawingBox d
  public void display (DrawingBox d, int x, int y, int r) {
    double theta;    // angle from 12:00
    int x1, y1;      // end point of a clock hand

    // Draw the clock in black.
    d.setColor(Color.black);
    d.drawOval(x-r, y-r, 2*r, 2*r);

    // Set the color to blue for the hands
    d.setColor(Color.blue);
    // Draw the minute hand.
    theta = 2*Math.PI*minute/60.0;
    x1 = x + (int)(r*Math.sin(theta));
    y1 = y - (int)(r*Math.cos(theta));
    d.drawLine(x, y, x1, y1);

    // Draw the hour hand.
    theta = 2*Math.PI*(hour+minute/60.0)/12.0;
    x1 = x + (int)(r*.8*Math.sin(theta));
    y1 = y - (int)(r*.8*Math.cos(theta));
    d.drawLine(x, y, x1, y1);
  }
}
