import CSLib.*;
import java.awt.*;

public class Clock {
  // Maintain a clock.
  // Author: Courtney Mickunas, November 21, 2000
  
  int minute;

  public Clock () {
    this(12, 0);
  }

  public Clock (int hour, int minute) {
    set(hour, minute);
  }

  // Obtain values for hour and minute from user input.
  public void getData () {
    InputBox in = new InputBox();
    in.setPrompt("What is the hour?");
    setHour(in.readInt());    
    in.setPrompt("What is the minute?");
    setMinute(in.readInt());    
  }

  // Return the current time as a string.
  public String toString() {
    if (getHour() == 0)
      return ("12:" + getMinute());
    else
      return (getHour() + ":" + getMinute());
  }

  public void setHour (int h) { set(h, getMinute()); }
  public void setMinute (int m) { set(getHour(), m); }

  public void set (int h, int m) {
    minute = (h * 60 + m) % (12 * 60);
    // minute is between -(12 * 60) and (12 * 60)
    if (minute < 0) minute = minute + (12 * 60);
  }

  public int getHour () {
    int hour = minute / 60;
    if (hour == 0)
      return 12;
    else
      return hour;
  }

  public int getMinute () { return (minute % 60); }

  // Compare the time of this clock to another.
  // Return true if this time is less than
  // the time of the other clock.
  public boolean priorTo (Clock c) {
    int oldhour = getHour(),
        oldchour = c.getHour();
    if (oldhour == 0) oldhour = 12;
    if (oldchour == 0) oldchour = 12;
    return (oldhour < oldchour ||
            oldhour == oldchour && getMinute() < c.getMinute()
           );
  }

  // Draw a clock with radius r in the center of the local DrawingBox
  public void display (DrawingBox d, int r) {
    display(d, d.getDrawableWidth()/2, d.getDrawableHeight()/2, r);
  }

  // Draw a clock with radius r at (x,y) in the DrawingBox d
  public void display (DrawingBox d, int x, int y, int r) {
    double theta;    // angle from 12:00
    int x1, y1;      // end point of a clock hand

    // Draw the clock in black.
    d.setColor(Color.black);
    d.drawOval(x-r, y-r, 2*r, 2*r);

    // Set the color to blue for the hands
    d.setColor(Color.blue);
    // Draw the minute hand.
    drawHand (d, x, y, r, getMinute()/60.0, 1.0);

    // Draw the hour hand.
    drawHand (d, x, y, r, (getHour()+getMinute()/60.0)/12.0, 0.8);
  }

  public void drawHand (DrawingBox d, int x, int y, int r, 
                        double fraction, double scale) {
    int x1, y1;
    double theta = 2*Math.PI*fraction;
    x1 = x + (int)(r*scale*Math.sin(theta));
    y1 = y - (int)(r*scale*Math.cos(theta)); 
    d.drawLine(x, y, x1, y1);
  }
}
