public class GradeDistribution {
  // Keep track of a grade distribution.
  // Author: Rachel N. Reingold, January 31, 2001

  // intervals stores beginning and ending of each
  //  group.  Groups are intervals[0]..intervals[1]-1,
  //  intervals[1]..intervals[2]-1, etc.
  //  Note that intervals.length is one more
  //  than the number of groups
  private int[] intervals;

  // counts stores the count for each group.  That is,
  //  counts[i] is the number of grades between
  //  intervals[i] and intervals[i+1]-1, inclusive.
  //  counts.length = intervals.length - 1
  private int[] counts;

  // Create distribution with given set of groups.
  // Argument is array to be copied into intervals.
  public GradeDistribution (int[] intervals) {
    this.intervals = new int[intervals.length];
    counts = new int[intervals.length-1];
    for (int i=0; i<intervals.length; i++)
      this.intervals[i] = intervals[i];
  }

  // Report new grade to be added to distribution
  public void insertGrade (int g) {
    // Ignore grades that are out of range
    if (g < intervals[0]) return;
    int i=0;
    while (i < counts.length && g >= intervals[i+1])
      i++;
    if (i == counts.length) return;
    else counts[i]++;
  }

  // Return number of groups
  public int numGroups () {
    return counts.length;
  }

  // Return count for given group
  public int groupCount (int gp) {
    if (gp < 0 || gp >= counts.length) return 0;
    return counts[gp];
  }

  // Return maximum count of any group (this is
  //  needed to calibrate sizes of graphs)
  public int maxCount () {
    // size of maximum group
    int max = 0;
    for (int i=0; i<counts.length; i++)
      if (max < counts[i]) max = counts[i];
    return max;
  }
}
