import CSLib.*;
import java.awt.*;

class SoftFrame {
  
  // A Frame for drawing lines and circles.
  // Author: Adrienne Baranowicz, December 2, 1999
  
  private
    Color[][] frameBuffer;
    int width, height;
  
  public SoftFrame (int w, int h) {
    width = w;
    height = h;
    frameBuffer = new Color[height][width];
  }
  
  private void setPixel (int col, int row, Color c) {
      if ((0 <= col && col < width)
          && (0 <= row && row < height))
        frameBuffer[row][col] = c;
    }
  
  public void clearSoftFrame () {
    for (int col = 0; col < width; col++)
      for (int row = 0; row < height; row++)
        frameBuffer[row][col] = Color.white;
  }
  
  public void displaySoftFrame (String s) {
    final int MAG = 4;  // Width & height of box used
                // to draw each pixel in the soft frame
    DrawingBox d = new DrawingBox(s);
    d.setDrawableSize(MAG*width, MAG*height);
    for (int row = 0; 
             row < frameBuffer.length; 
             row++) 
      for (int col = 0; 
               col < frameBuffer[row].length; 
               col++) {
        d.setColor(frameBuffer[row][col]);
        d.fillRectangle(col*MAG, row*MAG, MAG, MAG);
      }
  }
  
  public void drawHorizontalLine (int row) {
    for (int col = 0; col < width; col++)
      setPixel(col, row, Color.black);
  }
    
  public void drawVerticalLine (int col) {
    for (int row = 0; row < height; row++)
      setPixel(col, row, Color.black);
  }
  
  public void drawLongLine (double m, double b) {
    for (int col = 0; col < width; col++) {
      int row = (int)Math.round(m*col + b);
      setPixel(col, row, Color.black);
    }
  }
  
  private void swap (Point p1, Point p2) {
    int x = p1.x,    y = p1.y;
     p1.x = p2.x; p1.y = p2.y;
     p2.x = x;    p2.y = y;
  }
  
  public void drawLine1 (Point p1, Point p2) {
    int row, col;
  
    if (p1.x == p2.x) { // vertical line
      if (p2.y < p1.y)
        swap(p1, p2); // force p1.y <= p2.y
      for (row = p1.y; row <= p2.y; row++)
        setPixel(p1.x, row, Color.black);
    }
    else {
      double m = (p2.y-p1.y)/(double)(p2.x-p1.x),
             b = p1.y-m*p1.x;
      if (Math.abs(m) < 1.0) {
        if (p2.x < p1.x) // force p1 to left of p2
          swap(p1, p2);
        for (col = p1.x; col <= p2.x; col++) {
          row = (int)Math.round(m*col + b);
          setPixel(col, row, Color.black);
        }
      }
      else { // Math.abs(m) >= 1.0
        if (p2.y < p1.y) // force p1 above p2
          swap(p1, p2);
        for (row = p1.y; row <= p2.y; row++) {
          col = (int)Math.round ((row-b)/m);
          setPixel(col, row, Color.black);
        }
      }
    }
  }
  
  public void drawLine (Point p1, Point p2) {
  // This version of drawLine works only if 
  // p1.x<p2.x, p1.y<p2.y, and the slope of 
  // the line is positive and <= 1
    int dx = p2.x-p1.x;
    int dy = p2.y-p1.y;
    int p = 2*dy - dx;
    int x = p1.x,
        y = p1.y;
    setPixel(x, y, Color.black);
    for (x = p1.x+1; x <= p2.x; x++) {
      if (p > 0) y++;
      setPixel(x, y, Color.black);
      if (p < 0)
        p = p + 2*dy;
      else
        p = p + 2*(dy - dx);
    }
  }
  
  public void drawCircle1 (Point p0, int r) {
    int r2 = r*r;
    for (int x = -r; x <= r; x++) {
      int y = (int)Math.round(Math.sqrt(r2 - x*x));
      setPixel(p0.x+x, p0.y+y, Color.black);
      setPixel(p0.x+x, p0.y-y, Color.black);
    }
  }
  
  public void drawCircle (Point p0, int r) {
    int r2 = r*r;
    int y = 0;
    int x = r;
    do {
      setPixel(p0.x+x, p0.y+y, Color.black);
      setPixel(p0.x+y, p0.y+x, Color.black);
      setPixel(p0.x-y, p0.y+x, Color.black);
      setPixel(p0.x-x, p0.y+y, Color.black);
      setPixel(p0.x-x, p0.y-y, Color.black);
      setPixel(p0.x-y, p0.y-x, Color.black);
      setPixel(p0.x+y, p0.y-x, Color.black);
      setPixel(p0.x+x, p0.y-y, Color.black);
      y++;
      x = (int)Math.round(Math.sqrt(r2 - y*y));
    }
    while (y <= x);
  }
}

