import CSLib.*;
import java.awt.*;

class Mouse {
  // A mouse that can navigate a maze
  // Author: Allan M. Mickunas, September 22, 1996
  
  final int NORTH=0, EAST=1, SOUTH=2, WEST=3;

  private Maze theMaze;
  private boolean started = false; // true once the maze 
                                   // is entered.
  private Point location; // The location of this Mouse
  private int direction;  // The direction this Mouse is facing
  
  public Point tellLocation () {return location;}

  public int tellDirection () {return direction;}

  
  public Mouse(Maze m) {
    theMaze = m;
    // Where do I start?
    location = m.getStartLocation();
    // In what direction do I face initially?
    direction = m.getStartDirection(); 
  }
  
  public void makeMove() {
    // Make one move, using wall-to-the-right
    // algorithm from Chapter 1
    if (started) {
      if (!outside()) {
        turnRight();
        while (facingWall()) {
          turnLeft();
        }
        stepForward();
      }
    } else { 
      stepForward();
      started=true;
    }
  }
  
  private boolean outside () {
  // Am I outside the maze?
    return theMaze.outside(location);
  }
  
  private boolean facingWall () {
    return theMaze.checkWall(direction, location);
  }
  
  private void stepForward() {
    switch (direction) {
      case NORTH: location.y--; break;
      case EAST:  location.x++; break;
      case SOUTH: location.y++; break;
      case WEST:  location.x--; break;
    }
  }
  
  private void turnLeft() {
    direction = (direction+3) % 4;
  }
  
  private void turnRight() {
    direction = (direction+1) % 4;
  }
}


