import CSLib.*;

public class WindChill {
  // Produce a table of wind-chill indices
  // Author: Haim Reingold, March 16, 1994
  
  static final int 
    COLUMN_WIDTH = 5; //width of a column in the table
  
  private double windChillIndex (
         double temp,    // air temperature (deg F)
         double speed) { // wind speed (mph)
    if (speed <= 4)       //  little or no wind
      return temp;
    else if (speed <= 45) //  moderate wind
      return (91.4 - (10.45 + 6.69 * Math.sqrt(speed)
            - 0.447 * speed) * (91.4 - temp)/22.0);
    else                  //  high wind
      return (1.6 * temp - 55.0);
  }
  
  public void buildTable () {
    OutputBox out = new OutputBox("TABLE OF WIND-CHILL INDICES");
  
    // write the column headings
    out.print("degrees F:");
    for (int j = 5; j >= -6; j--)
      out.print(SFormat.sprintr(COLUMN_WIDTH, 10*j));  // column heading
    out.println();
    out.println();
  
    for (int i = 0; i <= 10; i++) {
      out.print(SFormat.sprintr(COLUMN_WIDTH, 5*i));
      out.print(" mph:"); // row label
      for (int j = 5; j >= -6; j--)
        out.print(SFormat.sprintr(COLUMN_WIDTH,
             (int)Math.round(windChillIndex(10*j,5*i))));
      out.println();
    }
  }
}
