import java.awt.*;

class Mouse {
  // A mouse that can navigate a maze
  // Author: Allan M. Mickunas, September 22, 1996
  
  final int NORTH=0, EAST=1, SOUTH=2, WEST=3;

  private Maze theMaze;
  private Point location; // The location of this Mouse
  private int direction;  // The direction this Mouse is facing
  private MouseAlgorithm ma;
  
  public Point tellLocation () {return location;}

  public int tellDirection () {return direction;}
  
  public Mouse (Maze m, MouseAlgorithm ma) {
    this.ma = ma;
    // Where do I start?
    location = m.getStartLocation();
    // In what direction do I face initially?
    direction = m.getStartDirection(); 
    theMaze = m;
  }
  
  public void makeMove () {
    ma.makeAlgorithmicMove(this);
  }
  
  public boolean outside () {
  // Am I outside the maze?
    return theMaze.outside(location);
  }
  
  public boolean facingWall () {
    return theMaze.checkWall(direction, location);
  }
  
  public void stepForward () {
    switch (direction) {
      case NORTH: location.y--; break;
      case EAST:  location.x++; break;
      case SOUTH: location.y++; break;
      case WEST:  location.x--; break;
    }
  }
  
  public void turnLeft () {
    direction = (direction+3) % 4;
  }
  
  public void turnRight () {
    direction = (direction+1) % 4;
  }
}
