import CSLib.*;
import java.awt.*;
import java.awt.event.*;

public class ClickableClock implements MouseListener {
  // Maintain a clock.
  // Author: Courtney Mickunas, November 21, 2000
  
  int hour, minute;
  DrawingBox d;

  public ClickableClock () {
    this(12, 0);
  }

  public ClickableClock (int h, int m) {
    set(h, m);
    d = new DrawingBox("ClickableClock Window");
    d.setSize(300, 320);
    d.addMouseListener(this);
  }

  public void mouseClicked (MouseEvent e) {
    if (e.getY() < d.getDrawableHeight()/2)
       set(hour, minute+5);
    else
      set(hour, minute+10);
    display();
   }

  public void mousePressed (MouseEvent e) { }
  public void mouseReleased (MouseEvent e) { }
  public void mouseEntered (MouseEvent e) { }
  public void mouseExited (MouseEvent e) { } 

  public void display () { display(1.0); }

  public void display (double size) {
    double theta;
    int x0, y0, x1, y1, radius;
    // Determine the center of the drawable field.
    x0 = d.getDrawableWidth()/2;
    y0 = d.getDrawableHeight()/2;
    // Determine the radius of the largest circle that fits.
    radius = (int)(Math.min(x0, y0) * size);

    // First clear the old clock.
    d.clear();

    // Now draw the clock.
    d.drawOval(x0 - radius, y0 - radius, 2*radius, 2*radius);
    // Set the color to blue for the hands
    d.setColor(Color.blue);
    // Draw the minute hand.
    theta = 2*Math.PI*minute/60.0;
    x1 = x0 + (int)(radius*Math.sin(theta));
    y1 = y0 - (int)(radius*Math.cos(theta));
    d.drawLine(x0, y0, x1, y1);

    // Draw the hour hand.
    theta = 2*Math.PI*(hour+minute/60.0)/12.0;
    x1 = x0 + (int)(radius*.8*Math.sin(theta));
    y1 = y0 - (int)(radius*.8*Math.cos(theta));
    d.drawLine(x0, y0, x1, y1);
  }

  // Obtain values for hour and minute from user input.
  public void getData () {
    InputBox in = new InputBox();
    in.setPrompt("What is the hour?");
    setHour(in.readInt());    
    in.setPrompt("What is the minute?");
    setMinute(in.readInt());
  }

  // Display the current time digitally.
  public void print () {
    OutputBox out = new OutputBox("ClickableClock");
    out.println("Time: " + hour + ":" + minute);
  }

  public void setHour (int h) { set(h, minute); }
  public void setMinute (int m) { set(hour, m); }

  public void set (int h, int m) {
    int totalMinutes = (h * 60 + m) % (12 * 60);
    // totalMinutes is between -(12 * 60) and (12 * 60)
    if (totalMinutes < 0) totalMinutes = totalMinutes + (12 * 60);
    hour = totalMinutes / 60;
    if (hour == 0) hour = 12;
    minute = totalMinutes % 60;
  }

  public int getHour () { return hour; }
  public int getMinute () { return minute; }

  public boolean priorTo (ClickableClock c) {
    return ((hour < c.hour) ||
            ((hour == c.hour) && (minute < c.minute)));
  }

  public boolean after (ClickableClock c) {
    return c.priorTo(this);
  }

  // Compare the time of this clock to another.
  // Return -1, 0, or +1 if this is less, equal,
  // or greater than the time of the other clock.
  public int compareTo (ClickableClock c) {
    if (hour < c.getHour())
      return -1;
    if (hour > c.getHour())
      return 1;
    // hour == c.getHour()
    if (minute < c.getMinute())
      return -1;
    if (minute > c.getMinute())
      return 1;
    return 0;
  }
}
