import CSLib.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class GUICalendar extends ClosableFrame
                   implements AdjustmentListener {
  
  // A Calendar, valid from 1/1/1000 to 12/31/2199
  // Author: Mark J. Mickunas, January 20, 1996
  
  Label banner;       // Month, year banner -- line 1
  Scrollbar select;   // Scrollbar -- line 2
  Panel days,         // days of week -- line 3
        pHead,        // contains lines 1, 2, 3 -- place in "North" of frame
        pBody;        // contains body -- placed in "Center" of frame
  
  int tallyMonth;     // in the range 0..13399

  Label[][] lDate = new Label[6][7];
  
  
  public GUICalendar () {
    super("Calendar");
  
  // Find out today's date.
    Calendar calendar = new GregorianCalendar();
    int month = calendar.get(Calendar.MONTH);// 0 through 11 !!!
    int year = calendar.get(Calendar.YEAR);
  
    tallyMonth = (year-1000)*12 + month;
  
  // Allocate and lay out the components
    banner = new Label();
    banner.setAlignment(Label.CENTER);
    banner.setBackground(Color.yellow);
    banner.setFont(new Font("TimesRoman",Font.BOLD, 16));
    banner.setForeground(Color.red);
  
    select = new Scrollbar(
                 Scrollbar.HORIZONTAL, // orientation
                 tallyMonth,           // initial value
                 1,                    // slider width
                 0,                    // minimum value
                 1200*12);             // maximum value

    days = new Panel();
    days.setLayout(new GridLayout(1,7));
    for (int i = 0; i<7; i++) {
      Panel p = new Panel();
      p.add(new Label(MyDate.DAY_NAME[i], Label.CENTER));
      days.add(p);
    }
    
    pHead = new Panel();
    pHead.setLayout(new GridLayout(3,1));
    pHead.add(banner);
    pHead.add(select); 
    pHead.add(days);

    pBody = new Panel();
    pBody.setLayout(new GridLayout(6,7));
    for (int i=0; i<6; i++)
      for (int j=0; j<7; j++) {
        lDate[i][j] = new Label("",Label.CENTER);
        lDate[i][j].setForeground(Color.blue);
        pBody.add(lDate[i][j]);
      }
  
    setLayout(new BorderLayout());
    add("North",pHead);
    add("Center",pBody);

  // Register this applet as the scrollbar listener.
    select.addAdjustmentListener(this);
    compute();
    pack();
    setVisible(true);
  }

  public void compute() {
  // Given tallyMonth, set the banner appropriately
  // and fill in the calendar body.

  // Compute month/year from the scrollbar position
  // and repaint the calendar body appropriately 
    int year = tallyMonth/12;
    int month = tallyMonth - 12*year + 1;
    year = year + 1000;
  
    banner.setText (MyDate.MONTH_NAME[month] + ", " + year);

  // Blank out the calendar
    for (int i=0; i<7; i++) 
      for (int j=0; j<6; j++)
        lDate[j][i].setText("");

  // Day of the week for the day 1 of the month
    int firstOfMonth = MyDate.dayOfWeek(month, 1, year);

  // The first label to be filled is lDaye[0][firstOfMonth]
    int x = firstOfMonth; 
    int y = 0;
 
  // Run through all days in the month
    for (int i = 1; 
         i <= MyDate.lastDayOfMonth(month, year); 
         i++) {
      
      lDate[y][x].setText(""+i);
      x++;
  
    // Is it time to move to the next line?
      if (x==7) {
        y++;
        x = 0;
      }
    }
    repaint();
  }

  public void adjustmentValueChanged (AdjustmentEvent e) {
  // Handle the scrollbar events
    tallyMonth = e.getValue();
    compute();
  }
}
