import CSLib.*;

public class MyDate {

// Class variables (all symbolic constants)

  public final static String[]
    DAY_NAME = {"Sun", "Mon", "Tue", "Wed", 
                "Thu", "Fri", "Sat"},
    MONTH_NAME = {"", 
            "January",   "February", "March",    "April", 
            "May",       "June",     "July",     "August", 
            "September", "October",  "November", "December"};

  public final static int
    SUNDAY=0,   MONDAY=1, TUESDAY=2, WEDNESDAY=3, 
    THURSDAY=4, FRIDAY=5, SATURDAY=6;

  public final static int
    JANUARY=1,   FEBRUARY=2, MARCH=3,     APRIL=4, 
    MAY=5,       JUNE=6,     JULY=7,      AUGUST=8, 
    SEPTEMBER=9, OCTOBER=10, NOVEMBER=11, DECEMBER=12;

// Class methods

  public static boolean dateIsLegal (int m, int d, int y) {
  // Determine whether month m, day d, year y
  // is a legal date on the Gregorian calendar;
  // date must be on or after Jan 1, 1 AD.
  
    return
      (1 <= y) &&                // year is okay
      (1 <= m) && (m <= 12) &&   // month is okay
      (1 <= d) &&                // day is...
      (d <= lastDayOfMonth(m, y));   //       ...okay
  }
  
  public static int lastDayOfMonth (int m, int y) {
  // Returns the number of days in month m of 
  // year y on the Gregorian calendar
  
    switch (m) {
      case 2: if (leapYear(y))
                return 29;
              else
                return 28;
      case 4:
      case 6:
      case 9:
      case 11: return 30;
      default: return 31;
    }
  }

  public static boolean leapYear (int y) {
  // Returns true if y is a leap year
  // according to the Gregorian calendar

    // y must be a multiple of 4 and NOT a century
    // year or must be a century year divisible by 400
  
    return (((y % 4) == 0) && ((y % 100) != 0))
           || ((y % 400) == 0);
  }
  
  public static int dayOfWeek (int m, int d, int y) {
  // Tell which day of the week is m/d/y
    return new MyDate(m,d,y).absolute()%7;
  }

  public static MyDate getNewDate () {
    MyDate d = new MyDate();
    d.getDate();
    return d;
  }

// Instance variables

  private int 
    m, // the month of this instance of Date
    d, // the day of this instance of Date
    y; // the year of this instance of Date
  
// Constructors

  public MyDate () {}
  
  public MyDate (int month, int day, int year) {
    m=month; d=day; y=year; 
  }
  
// Instance methods

  public boolean dateIsLegal () {
  // Determine whether this instance's month, day, year
  // is a legal date on the Gregorian calendar;
  // date must be on or after Jan 1, 1 AD.
    return dateIsLegal (m, d, y); 
  }

  public int lastDayOfMonth () {
  // Returns the number of days in this instance's
  // month and year on the Gregorian calendar
    return lastDayOfMonth (m, y); 
  }
  
  public boolean leapYear () {
  // Returns true if this instance's year is a leap year
  // according to the Gregorian calendar
    return leapYear(y);
  }
  
  public void getDate () {
    InputBox in = new InputBox();
    in.setPrompt("Enter month: ");
    m = in.readInt();
    in.setPrompt("Enter day: ");
    d = in.readInt();
    in.setPrompt("Enter year: ");
    y = in.readInt();
  }
  
  public String toString () {
    // Stringify the present date
    return m + "/" + d + "/" + y; 
  }
  
  public int absolute () {
  // Calculate the absolute date of this instance's date
  // according to the Gregorian calendar.
  
    return
      dayNumber()            // days this year
        + 365*(y-1)          // days in prior years
        + (y-1)/4            // \
        - (y-1)/100          // leap years
        + (y-1)/400;         // /
  }
  
  public int dayNumber () {
  // Calculate the day number in the year of this instance 
  // on the Gregorian calendar
  
    // Calculate day number assuming all months have 31 days
    int number = (m - 1) * 31 + d;
  
    //Correct for months beyond February
    if (m > 2) {
      number = number - ((4 * m + 23) / 10);
      if (leapYear())
        number = number + 1;
    };
  
    return number;
  }
  
  public int dayOfWeek () {
  // Tell which day of the week this Date is.
    return absolute()%7;
  }
}
