import CSLib.*;
import java.awt.*;

public class Clock extends Canvas {
  // Maintain a clock.
  // Author: Courtney Mickunas, November 21, 2000
  
  int hour,
      minute;

  public Clock () {
    this(12,0);
  }

  public Clock (int hour, int minute) {
    set(hour, minute);
  }

  // Return the current time as a string.
  public String toString () { return (hour + ":" + minute); }

  public void setHour (int h) { set(h, minute); }
  public void setMinute (int m) { set(hour, m); }

  public void set (int hour, int minute) {
    int totalMinutes = (hour * 60 + minute) % (12 * 60);
    // totalMinutes is between -(12 * 60) and (12 * 60)
    if (totalMinutes < 0) totalMinutes = totalMinutes + (12 * 60);
    this.hour = totalMinutes / 60;
    if (this.hour == 0) this.hour = 12;
    this.minute = totalMinutes % 60;
  }

  public int getHour () { return hour; }
  public int getMinute () { return minute; }

  // Compare the time of this clock to another.
  // Return true if this time is less than
  // the time of the other clock.
  public boolean priorTo (Clock c) {
    return (hour < c.hour ||
            hour == c.hour && minute < c.minute
           );
  }

  public boolean after (Clock c) {
    return c.priorTo(this);
  }

  // Draw a clock in this Canvas.
  public void paint (Graphics d) {
    int w = getWidth()-1,
        h = getHeight()-1;
    int x = w/2, 
        y = h/2;             // center at (x,y)
    int r = Math.min(x, y);  // radius

    // Draw the clock in black.
    d.setColor(Color.black);
    d.drawOval(x-r, y-r, 2*r, 2*r);
  
    // Set the color to blue for the hands
    d.setColor(Color.blue);
    // Draw the minute hand.
    drawHand(d, x, y, r, minute/60.0);
 
    // Draw the hour hand.
    drawHand(d, x, y, (int)(r*0.8), (hour+minute/60.0)/12.0);
  }

  private void drawHand (Graphics d, int x, int y, int r, double fraction) {
    int x1, y1;      // end point of a clock hand
    double theta;    // angle from 12:00

    theta = 2*Math.PI*fraction;
    x1 = x + (int)(r*Math.sin(theta));
    y1 = y - (int)(r*Math.cos(theta));
    d.drawLine(x, y, x1, y1);
  }
}
