import java.awt.*;
import java.awt.event.*;
import java.applet.*;

public class HilbertApplet extends Applet
                           implements ActionListener {

Panel topRow = new Panel();
TextField in = new TextField("1");
Hilbert david;

  public void init() {
    setLayout(new BorderLayout());
    topRow.add(new Label("Desired Order:"));
    topRow.add(in);
    in.addActionListener(this);
    add("North",topRow);
    david = new Hilbert(new Point(20,50));
  }

  public void paint (Graphics g) {
    Dimension d = this.getSize();
    david.paint(g, d);
  }

  public void actionPerformed(ActionEvent event) {
    int order=Integer.parseInt(in.getText().trim());
    david.setOrder(order);
    repaint();
  }

}

class Hilbert {

  // Draw Hilbert curves
  // Author: Kathryn Mickunas, May 23, 1996

  int order;
  int scaleFactor;
  Point nwOfWindow;     // drawing area starts here
  Polygon poly;
  int offset;

  Hilbert (Point p) {
    nwOfWindow = p;
    order = 1;
  }

  void setOrder (int o) {
    // Negative or zero order leads to infinite loop
    order = Math.max(o, 1);
  }

  int curvesize (int ord) {
    return Power.power(2, ord) - 1;
  }

  public void paint (Graphics g, Dimension d) {
    int windowSize = Math.min(d.width-nwOfWindow.x,
                           d.height-nwOfWindow.y) - 10;
    scaleFactor =
      (int)((windowSize*1.0) / curvesize(order));
    poly = new Polygon();
    hilbert(order);
    g.drawPolyline(poly.xpoints, poly.ypoints,
                                 poly.npoints);
  }

static final int NORTH=0, EAST=1, SOUTH=2, WEST=3;

  Point lastpoint;

  void move (int d) {
  // Move one unit in direction d from point lastpoint.

    switch (d) {
      case NORTH:lastpoint.y = lastpoint.y-scaleFactor;
                 break;
      case EAST: lastpoint.x = lastpoint.x+scaleFactor;
                 break;
      case SOUTH:lastpoint.y = lastpoint.y+scaleFactor;
                 break;
      case WEST: lastpoint.x = lastpoint.x-scaleFactor;
                 break;
    }
  }

  void hilbert (
     int i,      // order of Hilbert curve to be drawn
     int facing, // the compass direction we are facing
     int right,  // the compass direction to our right
     int behind, // the compass direction at our back
     int left){  // the compass direction to our left
  // Collect the coordinates of successive points on
  // the Hilbert curve of order i, starting at location
  // lastpoint, oriented as specifed by "local" compass

    if (i == 0)
      poly.addPoint(lastpoint.x, lastpoint.y);
    else {
      hilbert(i-1, left, behind, right, facing);
      move(right);
      hilbert(i-1, facing, right, behind, left);
      move(behind);
      hilbert(i-1, facing, right, behind, left);
      move(left);
      hilbert(i-1, right, facing, left, behind);
    }
  }

  void hilbert (int i) {
  // Draw a Hilbert curve of order i.
    lastpoint = new Point(nwOfWindow);
    hilbert(i, NORTH, EAST, SOUTH, WEST);
  }

}

class Power {

static int power (
           int k,  // number to be raised to a power
           int n) {// power to which it's to be raised
  if (n == 0)
    return 1;
  else {
    int t = power(k, n/2);
    if ((n % 2) == 0)
      return t*t;
    else
      return k*t*t;
  }
}
}

