import java.awt.*;
import CSLib.*;

//public class HilbertCurveClient {
    //HilbertCurve h = new HilbertCurve().draw();
  //}
//}

class HilbertCurve {

  static InputBox in;
  static Hilbert david;
  static int order;

  public static void main (String[] args) {
    in = new InputBox();
    in.setPrompt("Desired Order:");
    david = new Hilbert();
    order = in.readInt();
    david.hilbert(order);
    DrawingBox out = new DrawingBox(
                     "Hilbert curve of order " + order);
    david.draw(order, out);
  }
}

class Hilbert {

  // Draw Hilbert curves
  // Author: Kathryn Mickunas, May 23, 1996

  int order;
  Point nwOfWindow;     // drawing area starts here
  Polygon poly;
  int offset;

  Hilbert () {
    nwOfWindow = new Point(0,0);
    poly = new Polygon();
  }

  int curvesize (int ord) {
    return Power.power(2, ord) - 1;
  }

  public void draw (int order, DrawingBox b) {
    b.clearRect(0,0,b.getDrawableWidth(),
                    b.getDrawableHeight());	
    int windowSize = Math.min(b.getDrawableWidth()-nwOfWindow.x,
                           b.getDrawableHeight()-nwOfWindow.y) - 10;
    scaleFactor =
      (int)((windowSize*1.0) / curvesize(order));
    drawPolyline(b, scaleFactor, poly.xpoints, poly.ypoints,
                           poly.npoints);
  }

  void drawPolyline(DrawingBox b, int sf, int[] xpts, int[] ypts, int npts) {
    for (int i=0; i<npts-1; i++) {
      drawLine(b, sf, xpts[i], ypts[i], xpts[i+1], ypts[i+1]);
    }
  }

  void drawLine(DrawingBox b, int sf, int x1, int y1, int x2, int y2) {
    b.drawLine(sf*x1, sf*y1, sf*x2, sf*y2);
  }

  static final int NORTH=0, EAST=1, SOUTH=2, WEST=3;

  Point lastpoint;

  void move (int d) {
  // Move one unit in direction d from point lastpoint.

    switch (d) {
      case NORTH:lastpoint.y = lastpoint.y-1;
                 break;
      case EAST: lastpoint.x = lastpoint.x+1;
                 break;
      case SOUTH:lastpoint.y = lastpoint.y+1;
                 break;
      case WEST: lastpoint.x = lastpoint.x-1;
                 break;
    }
  }

  void hilbert (
     int i,      // order of Hilbert curve to be drawn
     int facing, // the compass direction we are facing
     int right,  // the compass direction to our right
     int behind, // the compass direction at our back
     int left){  // the compass direction to our left
  // Collect the coordinates of successive points on
  // the Hilbert curve of order i, starting at location
  // lastpoint, oriented as specifed by "local" compass

    if (i == 0)
      poly.addPoint(lastpoint.x, lastpoint.y);
    else {
      hilbert(i-1, left, behind, right, facing);
      move(right);
      hilbert(i-1, facing, right, behind, left);
      move(behind);
      hilbert(i-1, facing, right, behind, left);
      move(left);
      hilbert(i-1, right, facing, left, behind);
    }
  }

  void hilbert (int i) {
  // Draw a Hilbert curve of order i.
    lastpoint = new Point(nwOfWindow);
    hilbert(i, NORTH, EAST, SOUTH, WEST);
  }

}
