import java.awt.*;
import CSLib.*;

public class HilbertCurveClient {
  public static void main (String[] args) {
    new HilbertCurve().draw();
  }
}

class HilbertCurve {

  InputBox in;
  Hilbert david;
  int order;

  public void draw () {
    in = new InputBox();
    in.setPrompt("Desired Order:");
    order = in.readInt();
    david = new Hilbert(order);
    david.hilbert();
    DrawingBox out = new DrawingBox(
                     "Hilbert curve of order " + order);
    out.setDrawableSize(300, 300);
    david.draw(out);
  }
}

class Hilbert {

  // Draw Hilbert curves
  // Author: Kathryn Mickunas, May 23, 1996

  int order;
  Point nwOfWindow;     // drawing area starts here
  Polygon poly;

  final int OFFSET = 5;

  Hilbert (int order) {
    nwOfWindow = new Point(0,0);
    poly = new Polygon();
    this.order = order;
  }

  int curvesize (int ord) {
    return Power.power(2, ord) - 1;
  }

  public void draw (DrawingBox b) {
    b.clearRect(0,0,b.getDrawableWidth(),
                    b.getDrawableHeight());	
    int windowSize = b.getDrawableWidth()-2*OFFSET;
    int scaleFactor =
      (int)((windowSize*1.0) / curvesize(order));
    drawPolyline(b, scaleFactor, OFFSET, poly.xpoints, poly.ypoints,
                           poly.npoints);
  }

  void drawPolyline(DrawingBox b, int sf, int offset,
                    int[] xpts, int[] ypts, int npts) {
    for (int i=0; i<npts-1; i++) {
      b.drawLine(sf*xpts[i]+offset, sf*ypts[i]+offset,
                 sf*xpts[i+1]+offset, sf*ypts[i+1]+offset);
    }
  }

  static final int NORTH=0, EAST=1, SOUTH=2, WEST=3;

  Point lastpoint;

  void move (int d) {
  // Move one unit in direction d from point lastpoint.

    switch (d) {
      case NORTH:lastpoint.y = lastpoint.y-1;
                 break;
      case EAST: lastpoint.x = lastpoint.x+1;
                 break;
      case SOUTH:lastpoint.y = lastpoint.y+1;
                 break;
      case WEST: lastpoint.x = lastpoint.x-1;
                 break;
    }
  }

  void hilbert (
     int i,      // order of Hilbert curve to be drawn
     int facing, // the compass direction we are facing
     int right,  // the compass direction to our right
     int behind, // the compass direction at our back
     int left){  // the compass direction to our left
  // Collect the coordinates of successive points on
  // the Hilbert curve of order i, starting at location
  // lastpoint, oriented as specifed by "local" compass

    if (i == 0)
      poly.addPoint(lastpoint.x, lastpoint.y);
    else {
      hilbert(i-1, left, behind, right, facing);
      move(right);
      hilbert(i-1, facing, right, behind, left);
      move(behind);
      hilbert(i-1, facing, right, behind, left);
      move(left);
      hilbert(i-1, right, facing, left, behind);
    }
  }

  void hilbert () {
  // Draw a Hilbert curve of order "order".
  // Result goes into Polygon poly
    lastpoint = new Point(nwOfWindow);
    hilbert(order, NORTH, EAST, SOUTH, WEST);
  }

}
