class SolvePP {
  
static void read2DArray (double[][] A) {
  for (int i=0; i<A.length; i++)
    for (int j=0; j<A[i].length; j++)
      A[i][j]=Keyboard.readDouble();
}

static void readArray (double[] B) {
  for (int i=0; i<B.length; i++)
    B[i]=Keyboard.readDouble();
}

static void write2DArray (double[][] A) {
  for (int i=0; i<A.length; i++) {
    for (int j=0; j<A[i].length; j++)
      System.out.print(A[i][j] + " ");
    System.out.println();
  }
  System.out.println();
}

static void writeArray (double[] X) {
  for (int i=0; i<X.length; i++)
    System.out.print(X[i] + " ");
  System.out.println();
}

static void eliminate (
            double[][] A, double[] B, int i
            ) {
  // Eliminate variable xi from system whose 
  // coefficients are in  A[0][0]..A[i][i],
  // with right-hand sides B[0]..B[i]
  double m;

  for (int j=0; j<i; j++) {
    m = A[j][i] / A[i][i];
    B[j] = B[j] - m*B[i];
    for (int k=0; k<i; k++)
      A[j][k] = A[j][k] - m*A[i][k];
  }
}

static void computeXi (
          double[][] A, // Using A[0][0]..A[i-1][i-1]
          double[] B,   // and B[i],
          double[] X,   // and X[0]..X[i-1], compute
          int i         // X[i] using equation (11.1)
          ) {
  double t = B[i];
  for (int j=0; j<i; j++)
    t = t - A[i][j]*X[j];
  X[i] = t/A[i][i];
}

static void swap (double[] x, int i, int j) {
  double temp = x[i];
  x[i] = x[j];
  x[j] = temp;
}

static void swapRows (
            double[][] A, int w, int j, int i
            ) {
  double temp;
  // Swap first w+1 elements of rows j and i of A
  for (int k=0; k<=w; k++) {
    temp = A[j][k];
    A[j][k] = A[i][k];
    A[i][k] = temp;
  }
} 

static void partialPivot (
            double[][] A, double[] B, int i
            ) {
  // Swap rows so that A[i][i] is non-zero
  int j = i-1;
  while ((j >= 0) && (A[j][i] == 0))
    // Find row with non-zero first element
    j--;
  if (j == -1) {
    new ErrorBox("Pivoting failed:  system unsolvable");
    Timer.pause(3000);  // let user read message
    System.exit(1);
  }
  swapRows(A, i, j, i);
  swap(B, j, i);
}

static void solve (
       double[][] A, // Solve system whose coeffic-
                     // ients are in A[0][0]..A[i][i],
       double[] B, // and right-hand sides B[0]..B[i]
       double[] X, // Place solutions in X[0]..X[i]
       int i) {
  if (i == 0) // one-equation system - solve directly
    X[0] = B[0]/A[0][0];
  else {
    if (A[i][i] == 0)
      partialPivot(A, B, i); // swap rows so A[i][i]!=0
    eliminate(A, B, i);  // eliminate Xi; new system 
                         // is in A[0][0]..A[i-1][i-1] 
                         // and B[0]..B[i-1]
    solve(A, B, X, i-1);  // Fill in X[0]...X[i-1]
    computeXi(A, B, X, i);// calculate X[i] from A, B,
                          // and X[0]...X[i-1]
  }
}

public static void main (String[] args) {
  int n = 3;
  double[][] A = new double[n][n];
  double[]   B = new double[n],
             X = new double[n];

//  System.out.print("Input A[0][0], A[0][1],"+
//"..., A[1][0], ..., A[n-1][n-1]\n");
  read2DArray(A);
  readArray(B);
  solve(A, B, X, 2);
  System.out.print("Solution vector is: ");
  writeArray(X);
  System.out.println();
}

}

