import java.awt.*;
import java.awt.event.*;
import java.applet.*;

class QueryManager implements ActionListener, 
                  ItemListener {

  // Set up a query manager for a CD Database
  // Author: Samuel N. Kamin, June 1, l996

  Applet app;    // The applet containing this form
  
  CDDatabase db; // The database being queried
  
  Query q;       // The query entered by the user

  
  // Components of the applet window
  TextField artist, title;
  Checkbox caseMatters, prefix;
  Button submitButton, resubmitButton;
  Label result;
  
  public QueryManager (CDDatabase db, Applet app) {
    this.app = app;
    this.db = db;
    q = new Query();
  }
  
  public void createForm () {

    // Lay out the screen
    app.setLayout(new GridLayout(6,1));

    app.add(new Label("CD Search Form", Label.CENTER));
    app.add(
      new Label("Fill in part of artist's name "
                + "and/or CD's title", Label.CENTER));

    Panel p = new Panel();
    artist = new TextField(20);
    p.add(new Label("Artist: "));
    p.add(artist);
    title = new TextField(20);
    p.add(new Label("Title: "));
    p.add(title);
    app.add(p);

    p = new Panel();
    caseMatters = new Checkbox();
    p.add(new Label("Case matters: "));
    p.add(caseMatters);
    prefix = new Checkbox();
    p.add(new Label("Match prefix only: "));
    p.add(prefix);
    app.add(p);

    p = new Panel();
    submitButton = new Button("Submit query");
    p.add(submitButton);
    resubmitButton = new Button("Resubmit query");
    p.add(resubmitButton);
    app.add(p);

    result = new Label("", Label.CENTER);
    app.add(result);
    
    // Register listeners
    artist.addActionListener(this);
    title.addActionListener(this);
    submitButton.addActionListener(this);
    resubmitButton.addActionListener(this);
    caseMatters.addItemListener(this);
    prefix.addItemListener(this);
  }
  
  public void actionPerformed (ActionEvent e) {
    // Four actions matter 
    // typing Enter in the artist or title textfields 
    // clicking on the submit or resubmit button
    //
    // The first three have the same effect, 
    //           to submit the query.
    q.setArtist(artist.getText());
    q.setTitle(title.getText());
    if (e.getSource() == resubmitButton)
      redoQuery();
    else
      doQuery();
  }
  
  public void itemStateChanged (ItemEvent e) {
    if (e.getSource() == caseMatters)
      q.setCase(caseMatters.getState());
    else if (e.getSource() == prefix)
      q.setPrefix(prefix.getState());
  }

  private void redoQuery() {
    CDRecord qresult = db.redoQuery(q);
    if (qresult == null) // query failed
      result.setText("No more matches");
    else
      result.setText("Next match: " + qresult.toString());
  }

  private void doQuery() {
    CDRecord qresult = db.doQuery(q);
    if (qresult == null) // query failed
      result.setText("No matching CD's; reformulate query");
    else
      result.setText("First match: " + qresult.toString());
  }

}
