import CSLib.*;
import java.io.*;

public class CDBaseApplication {
  // CD Database Search
  // Author: Samuel N. Kamin, June 1, l996

  private static BufferedReader dbreader, sysin;

  private CDDatabase db;
  private Query q;
  private CDRecord qresult;

  public CDBaseApplication () throws IOException {
    // open file "MyCDs" for stream character reading
    dbreader = new BufferedReader(new FileReader("MyCDs"));

    // Make System.in readable using readLine()
    sysin = new BufferedReader(new InputStreamReader(System.in)); 

    // call CDDatabase passing dbstream to create database
    db = new CDDatabase(dbreader);

    // create an unfilled query
    q = new Query();
  }

  public void queryLoop () throws IOException {

    while (true) {
      // Prompt for query parameters.
      System.out.print("Enter part of artist's name: ");
      String artist = sysin.readLine();
      System.out.print("Enter part of title: ");
      String title = sysin.readLine();
      System.out.print("Case matters (y,n)? ");
      boolean caseMatters = getYesNo();
      System.out.print("Match prefix only (y,n)? ");
      boolean prefix = getYesNo();

      // Set query parameters.
      q.setArtist(artist);
      q.setTitle(title);
      q.setCase(caseMatters);
      q.setPrefix(prefix);

      // Perform the query.
      qresult = db.doQuery(q);

      if (qresult == null) {
        System.out.println("Sorry, there were no matches for your query.");
      }

      while (qresult != null) {
        System.out.println("Result: " + qresult.toString());

        // Next query.
        System.out.print("Do you want to see the next match? ");
        if (!getYesNo()) break;
        qresult = db.redoQuery(q);
        if (qresult == null)
          System.out.println("No more matches for this query.");
      }

      System.out.print("Try again? ");
      if (!getYesNo()) break;
    }
  }

  private boolean getYesNo () throws IOException {
    while (true) {
      String response = sysin.readLine();
      if (response.equalsIgnoreCase("y") 
          || response.equalsIgnoreCase("yes"))
        return true;
      if (response.equalsIgnoreCase("n") 
          || response.equalsIgnoreCase("no"))
        return false;
      System.out.print("Please respond yes or no: ");
    }
  }
}
