import java.io.*;

public class MailMerge {
  // A Mail Merge Application
  // Author: Mary Angela McDermott, December 30, 1996

  BufferedReader template, mailList;
  PrintWriter out;

  public void openFiles (String fn) throws IOException {

    template = new BufferedReader(
                   new FileReader(fn + ".template"));
    mailList = new BufferedReader(
                   new FileReader(fn + ".list"));

    out = new PrintWriter(
              new BufferedWriter(
                  new FileWriter(fn + ".out")));
  }

  private StringBuffer readUpto (BufferedReader br, 
                                char delim) 
                                throws IOException {

    StringBuffer sb = new StringBuffer();
    int inputchar;
  
    while (true) {
      inputchar = br.read();
      if (inputchar == -1 || inputchar == delim) return sb;
      sb.append((char)inputchar);
    }
  }
  
  private StringBuffer[] st = new StringBuffer[10];

  private int readTemplate() throws IOException {
    int n;

    // Read the first portion of template (which may
    // be empty, if there is a % as the first character).
    st[0] = readUpto(template, '%');
    // Read the remaining portions of template.
    for (n=1; n<10; n++) {
      st[n] = readUpto(template, '%');
      if (st[n].length() == 0) return n;
    }
    return n;
  }

  public void merge () throws IOException { 
    StringBuffer sm;

    int n = readTemplate();

    // Read the first field
    sm = readUpto(mailList, '#');

    while (true) {
      // If no more fields
      if (sm.length() == 0) return;

      // Interleave template portions with fields.
      out.print(st[0]);
      for (int i=1; i<n; i++) {
        mailList.read();  // get past newline
        out.print(sm);    // print field
        out.print(st[i]); // print next template portion
        sm = readUpto(mailList, '#');
      }
      out.println("----------------------------------");
    }
  }
  
  public void closeFiles () { 
    out.close();
  }
}
