import java.awt.*;

class Referee implements GameConstants {

  private final static String[] 
    YOUR_MOVE = {"It's your move", "It's Reversi's move"},
    STYMIED = {"You're stymied; it's still Reversi's move",
              "Reversi's stymied; it's still your move"};

  private Board theBoard;  // The actual game board
  private int whoseTurn;   // Tells whose turn it is.
  private Display dev;     // The display mechanism
  private Announce msg;    // The message mechanism
  
  Referee (Display dev, Announce msg) {
    this.dev = dev;
    this.msg = msg;
    theBoard = new Board();
    whoseTurn = PERSON;
    msg.setMessage(YOUR_MOVE[whoseTurn]); 
  }

  // Tell whose turn it is.
  public int whoseTurn () { return whoseTurn; }

  // Tell what occupies a given square.
  public int status (int row, int col) {
    return theBoard.field[row][col];
  }

  // Duplicate the game board.
  public Board duplicate () {
    return theBoard.duplicate();
  }

  // Tell if a move is legal or not.
  public boolean isLegal (int which, Point move) {
    if (whoseTurn != which) return false;
    return ( theBoard.isLegal(which, move) );
  }
  
  // Tell if the game is over or not.
  // The game is over if no one has a move.
  public boolean gameOver () {
    return (theBoard.howManyMoves(PERSON)
         + theBoard.howManyMoves(PLAYER))==0;
  }

  // Make a suggested move, if it is legal.
  public void makeMove (int participant, Point move) {
    if (!isLegal(participant, move)) {
      msg.setMessage("Illegal move.");
      return;
    }
    theBoard.makeMove(participant, move); 
    if (gameOver()) {
      scoreGame(); 
    } else {
      // take turns ...
      whoseTurn = theBoard.opposite(whoseTurn);
      if (theBoard.howManyMoves(whoseTurn) > 0) {
        msg.setMessage(YOUR_MOVE[whoseTurn]); 
      } else {  // ... unless the opponent is stymied
        msg.setMessage(STYMIED[whoseTurn]);
        whoseTurn = theBoard.opposite(whoseTurn);
      }
    }
    // Update the display
    dev.reShow();
  }
  
  // Tally  & report the final score of the game.
  private void scoreGame () {
    int
      v1 = theBoard.count(PERSON),
      v2 = theBoard.count(PLAYER);
  
    msg.setMessage("Game Over");
    if (v1<v2)
      msg.setMessage("You lose!");
    else if (v2<v1)
      msg.setMessage("You win!");
    else 
      msg.setMessage("It's a tie!");
  }
}
